// XCmyk.cpp  Version 1.0
//
// Author:  Hans Dietrich
//          hdietrich2@hotmail.com
//
// Description:
//     XCmyk.cpp implements functions to convert RGB to CMYK and CMYK to RGB.
//
// History
//     Version 1.0 - 2003 July 5
//     - Initial public release
//
// This software is released into the public domain.  You are free to use it 
// in any way you like.
//
// This software is provided "as is" with no expressed or implied warranty.  
// I accept no liability for any damage or loss of business that this software 
// may cause.
//
///////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "XCmyk.h"

///////////////////////////////////////////////////////////////////////////////
//
// The algorithms for these routines were taken from:
//     http://www.neuro.sfc.keio.ac.jp/~aly/polygon/info/color-space-faq.html
//
// RGB --> CMYK                              CMYK --> RGB
// ---------------------------------------   --------------------------------------------
// Black   = minimum(1-Red,1-Green,1-Blue)   Red   = 1-minimum(1,Cyan*(1-Black)+Black)
// Cyan    = (1-Red-Black)/(1-Black)         Green = 1-minimum(1,Magenta*(1-Black)+Black)
// Magenta = (1-Green-Black)/(1-Black)       Blue  = 1-minimum(1,Yellow*(1-Black)+Black)
// Yellow  = (1-Blue-Black)/(1-Black)
//


///////////////////////////////////////////////////////////////////////////////
// RGB2CMYK
void RGB2CMYK(BYTE r, BYTE g, BYTE b, BYTE& c, BYTE& m, BYTE& y, SHORT& k)
{

	short R, G, B;
	char C, M, Y;

	R = (short) r;
	G = (short) g;
	B = (short) b;

	C = (B /8) ;
	M = (G /8) ;
	Y = (R /8) ;
	k = C*1024 + M*32 + Y;

	c = (BYTE) C;
	m = (BYTE) M;
	y = (BYTE) Y;
}

///////////////////////////////////////////////////////////////////////////////
// CMYK2RGB
COLORREF CMYK2RGB(BYTE c, BYTE m, BYTE y, SHORT k)
{
	BYTE r, g, b;
	COLORREF rgb;
	
	short R, G, B;
	char C, M, Y;

	C = (char) c;
	M = (char) m;
	Y = (char) y;
	k = C*1024 + M*32 + Y;

	R = (k % 32)*8;
	G = ((k/32) % 32)*8;
	B = ((k/1024) % 32)*8;

	r = (BYTE) R;
	g = (BYTE) G;
	b = (BYTE) B;

	rgb = RGB(r,g,b);

	return rgb;
}
